<h{{ level }} class="mb-3">
{% if content|is_component %}
  {# NOTE: `content` is safe because it has been pre-rendered #}
  {{ content.model_dump_html()|safe }}
{% else %}
  {{ content }}
{% endif %}
</h{{ level }}>
